package com.njcn.mapper.configuration;

import com.njcn.pojo.configuration.ConstantValue;
import org.apache.ibatis.annotations.Param;
import tk.mybatis.mapper.common.Mapper;

import java.util.List;

/** @Author denghuajun
 * @Description //TODO 定值管理通用mapper
 * @Date  2018/10/15 21:30
 **/
public interface ConstantValueMapper extends Mapper<ConstantValue> {
    List<ConstantValue> quarryOutValueBase(@Param("lineIndex")long lineIndex);
    List<ConstantValue> quarryInValueBase(@Param("devIndex")long devIndex);
    int quarryOutCount(@Param("devtype")String devtype,@Param("lineIndex")long lineIndex);
    int deleteWord(@Param("devtype")String devtype);
    int deleteValue(@Param("devtype")int devtype,@Param("type")String type);//删除内部定值
    int deleteOutValue(@Param("lineIndex")int lineIndex,@Param("type")String type);//删除外部定值
    int updateOut(@Param("cpuNo")int cpuNo,@Param("type")String type,@Param("sort")int sort,@Param("value")Float value);
    int updateIn(@Param("cpuNo")int cpuNo,@Param("type")String type,@Param("sort")int sort,@Param("value")int
            value);
    ConstantValue getConvId(@Param("cpuNo")int cpuNo,@Param("type")String type,@Param("sort")int sort);
}
